@@ -1,4 +1,4 @@
-lpc <- function(X,h, t0=mean(h),  x0,   mult=1, way = "two", scaled=TRUE,  weights=1, pen=2,
+lpc <- function(X,h, t0=mean(h),  x0=1,   way = "two", scaled=TRUE,  weights=1, pen=2,
               depth=1, control=lpc.control()){
   iter    <-  control$iter
   boundary <- control$boundary
@@ -6,48 +6,75 @@ lpc <- function(X,h, t0=mean(h),  x0,   mult=1, way = "two", scaled=TRUE,  weigh
   thresh  <- control$pruning.thresh
   rho0    <- control$rho0
   cross   <- control$cross
-  Xi       <- as.matrix(X)
-  N        <- dim(Xi)[1]
-  d        <- dim(Xi)[2]
+  mult    <- control$mult
+
+  Xi      <- as.matrix(X)
+  N       <- dim(Xi)[1]
+  d       <- dim(Xi)[2]
   if (N %% length(weights) !=0){
        stop("The length of the vector of weights is not a multiple of the sample size.")
   } else {
       weights <-rep(weights, N %/%   length(weights))
   }     
-  X1       <- matrix(0,mult,d) # collects starting points for branches of depth 1.
-  X2<-X3   <- matrix(0,0,d)    # collect starting points for branches of depth 2 and 3.
-  S2<-S3   <- matrix(0,0,d)    # collects candidates for starting points of depth 2 and 3.
-  saveall  <- matrix(0,0,d)    # stores the LPC
+ 
   countb   <- 0                # counter for branches 17/07/2007
   Lambda   <- matrix(0,0,5)    # stores the parameters (lambda, countb) as well as  depth, order, and  initialization number of the corresponding branch.
   s1       <- apply(Xi, 2, function(dat){ diff(range(dat))})  # range
   if (missing(h)){ if (!scaled){h   <- s1/10} else {h<- 0.1}} # bandwidth by default: 10% of range
   if (length(h)==1){h <- rep(h,d)}
+  ms.h    <- if (!is.null(control$ms.h)){control$ms.h} else {h}
+  ms.sub  <- control$ms.sub 
   if (scaled){        # scales the data to lie by its range
         Xi <- sweep(Xi, 2, s1, "/")
    } 
   if (d==1){
       stop("Data set needs to consist of at least two variables!")
-  } #else if (d > 2 && depth > 1){            # removed 05/05/11
-    #  cat("Dimension of data set enforces depth=1! \n")
-    #  depth <- 1
-    #}
-  if (!missing(x0)){
-      x0 <- matrix(x0, ncol=d, byrow=TRUE)
+  } 
+
+  # Selecting and scaling starting points:
+  if (length(x0)==1 && x0==1){
+    
+      n  <- sample(N,1);
+      x0 <- matrix(ms.rep(Xi, Xi[n,],ms.h, plotms=0)$final, nrow=1)
+      
+   }  else if (length(x0)==1 && x0==0 ){
+          if (N<= ms.sub) {
+              sub<- 1:ms.sub
+          }  else {    
+              Nsub <- min(max(ms.sub, floor(ms.sub*N/100)), 10*ms.sub)
+              #print(Nsub)
+              sub <- sample(1:N, Nsub)
+          }
+          x0<- ms(Xi, ms.h, subset=sub, plotms=0)$cluster.center
+   }  else {
+        if (is.null(x0)){
+             if (is.null(mult)){stop("One needs to allow for at least one starting point.")}
+             x0<- matrix(0, nrow=0, ncol=d)
+        } else {    
+             x0 <- matrix(x0, ncol=d, byrow=TRUE)
+        }   
       if (scaled){ x0 <- sweep(x0, 2, s1, "/") } # scales the starting point just as the scaled data
-      if (dim(x0)[1] < mult) {n <- runif(mult-dim(x0)[1],1,N+1)%/%1; x0 <- rbind(x0,Xi[n,])}  # creates the appropriate number of random starting points
-  } else {
-       n <- runif(mult,1,N+1)%/%1;     
-       x0 <- matrix(Xi[n,], length(n),d)# corrected 16/10          
   }
 
  
+  if(!is.null(mult)){  
+      if (dim(x0)[1] < mult) {n <- runif(mult-dim(x0)[1],1,N+1)%/%1; x0 <- rbind(x0,Xi[n,])} #
+      if (dim(x0)[1] > mult) {x0<-x0[1:mult]} 
+  }
+  x0       <- as.matrix(x0)    # putting in matrix format; just in case.... 
+  mult     <- dim(x0)[1]       # final number of starting points used
+  X1       <- matrix(0,mult,d) # collects starting points for branches of depth 1.
+  X2<-X3   <- matrix(0,0,d)    # collect starting points for branches of depth 2 and 3.
+  S2<-S3   <- matrix(0,0,d)    # collects candidates for starting points of depth 2 and 3.
+  saveall  <- matrix(0,0,d)    # stores the LPC
+
+  
   gapsize <- 1.5 # Note: In the original paper, this was set to 2, in order to avoid higher-depth branches to run back into the main branch. 
                  # This  effect is now partly achieved through "jweights", see below.
   
   for (j in 1:mult){
     
-     xo           <- x0[j,]                       # defines appropraite starting point for the jth curve
+     xo           <- x0[j,]                       # defines appropriate starting point for the jth curve
      X1[j,]       <- xo                           # adds starting point to list  
      curve0       <-  followx(Xi, xo, h, t0, iter, way, weights, pen, phi =1, 0,rho0,  boundary, convergence.at,  cross ) # computes LPC of depth 1
      saveall      <- rbind(saveall,curve0[[1]])	  # stores LPC
@@ -55,6 +82,7 @@ lpc <- function(X,h, t0=mean(h),  x0,   mult=1, way = "two", scaled=TRUE,  weigh
      Lambda       <- rbind(Lambda, cbind(curve0[[6]], countb,1,1,j))
      countb       <- countb + 1
      dimnames(Lambda)[[2]]<- c("lambda", "branch", "depth", "order", "init")
+
      
      if (depth >1 && l>=1 ){							  # constructs branches of depth 2.
 
@@ -129,6 +157,8 @@ lpc <- function(X,h, t0=mean(h),  x0,   mult=1, way = "two", scaled=TRUE,  weigh
      } # end if (depth)
   } # end for (j)
 
+#print(X1)
+  
    # if (d==2){
       starting.points <- rbind(X1,X2,X3) 
       dimnames(starting.points)<-list(c(rep(1, dim(X1)[1]), rep(2, dim(X2)[1]), rep(3, dim(X3)[1]) ), NULL)
@@ -139,6 +169,7 @@ lpc <- function(X,h, t0=mean(h),  x0,   mult=1, way = "two", scaled=TRUE,  weigh
    #}
 
 
+  
   fit<-  c(LPC=list(saveall),
            Parametrization= list(Lambda),
            h=list(h),
@@ -148,7 +179,7 @@ lpc <- function(X,h, t0=mean(h),  x0,   mult=1, way = "two", scaled=TRUE,  weigh
            scaled=list(scaled),
            weights=list(weights),
            control=list(control),
-           Misc=list(list( rho=curve0[[4]], scaled.by = if (scaled) s1 else rep(1,d), adaptive.band=curve0[[7]] ))
+           Misc=list(list( rho=curve0[[4]], scaled.by = if (scaled) s1 else rep(1,d), adaptive.band=curve0[[7]] , angles=curve0[[3]]))
            ) 
    class(fit)<-"lpc"
    fit
